//==============================================================================
//Copyright(c), Meilhaus Electronic GmbH
//              http://www.meilhaus.de
//              support@meilhaus.de
//
//Source File : uMain.pas
//Description : Example program for reading single values from AI subdevices
//              - Open main driver (ME-iDS)
//              - Go through the list of all devices
//              - Find AI subdevices and information about channels and ranges
//              - Add AI subdevices to the combobox, so the user can select one
//              - Read data from selected subdevice when user clicks on button "Read Data"
//              - Close main driver when program is closed
//==============================================================================
unit uMain;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, ExtCtrls, meIDS, ComCtrls, Math;

type
  TAnalogRange = class;
  TAiSubdevice = class;

  TfrmMain = class(TForm)
    lbl1: TLabel;
    btnClearOutput: TButton;
    memoInfo: TMemo;
    btnClose: TButton;
    btnReadData: TButton;
    lbl3: TLabel;
    coBoxSubdevs: TComboBox;
    grpBox1: TGroupBox;
    btnAbout: TButton;
    coBoxChannel: TComboBox;
    lbl2: TLabel;
    lbl4: TLabel;
    coBoxRange: TComboBox;
    btnReset: TButton;
    procedure btnCloseClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure btnClearOutputClick(Sender: TObject);
    procedure btnReadDataClick(Sender: TObject);
    procedure btnAboutClick(Sender: TObject);
    procedure coBoxSubdevsChange(Sender: TObject);
    procedure btnResetClick(Sender: TObject);
    procedure FormShow(Sender: TObject);
  private
    { Private-Deklarationen }
    openErrSuccess: Integer;
    subDeviceList: TList;
    lineNumber: Integer;
    function MeVersionToString(version: Integer): string;
    procedure DisplayVersionLibrary();
    procedure DisplayVersionMainDriver();
    function GetDevicePlugged(idxDevice: Integer; var plugged: Integer): Integer;
    procedure DisplaySubdevices();
    procedure GetAiSubdevices(numDevices: Integer);
    procedure GetSubdevicesByType(idxDevice: Integer; subdeviceType: Integer);
    procedure AddAiSubdevice(devName: string; idxDevice: Integer;
      idxSubdevice: Integer; subdeviceType: Integer);
    procedure AddAnalogRanges(aiSubdevice: TAiSubdevice);
    function GetSubDevTypeName(subdeviceType: Integer): string;
    procedure FillSubdeviceProperties(idxSelectedSubdev: Integer);
    function ReadSingleValue(aiSubdevice: TAiSubdevice; idxChannel: Integer; var value: Integer): Integer;
    procedure CleanupSubdeviceCollection();
    procedure OpenMEiDSDriver();
    procedure CloseMEiDSDriver();
    procedure DisplayErrorMessage(ErrMsg: string; meError: Integer);
    function ConfigureSubdeviceForInput(aiSubdevice: TAiSubdevice; idxChannel, idxRange: Integer): Integer;
    procedure AddAnalogRange(aiSubdevice: TAiSubdevice; idxRange: Integer);
    procedure FillSubdeviceChannels(aiSubdevice: TAiSubdevice);
    procedure FillSubdeviceRanges(aiSubdevice: TAiSubdevice);
    function ConvertDigitalToPhysical(aiSubdevice: TAiSubdevice; idxRange: Integer;
      value: Integer; var valAnalog: Double): Integer;
    procedure DisplayValue(aiSubdevice: TAiSubdevice; idxRange: Integer; valDigital: Integer; valPhysical: Double);
  public
    { Public-Deklarationen }
  end;


//-------------------------------------------------
{TAISubDevice}
  TAnalogRange = class(TCollectionItem)
  private
    FRangeUnit: Integer;
    FDigitalMax: Integer;
    FPhysicalMin: Double;
    FPhysicalMax: Double;
    FDisplayRange: string;                                                      
    FDisplayUnit: string;                                                       
    procedure SetDisplayText;
    function RoundPrecise(Value: Double; Decimals: Integer): Double;
  published
    property RangeUnit: Integer read FRangeUnit write FRangeUnit;
    property DigitalMax: Integer read FDigitalMax write FDigitalMax;
    property PhysicalMin: Double read FPhysicalMin write FPhysicalMin;
    property PhysicalMax: Double read FPhysicalMax write FPhysicalMax;
    property DisplayRange: string read FDisplayRange;
    property DisplayUnit : string read FDisplayUnit;
  end;

  TAiSubdevice = class(TCollection)
  private
    FDeviceName: string;
    FIndexDevice: Integer;
    FIndexSubdevice: Integer;
    FNumberOfChannels: Integer;
    function GetRange(Index: Integer): TAnalogRange;
  public
    function AddRange(AUnit: Integer; APhysicalMin, APhysicalMax: Double; ADigitalMax: Integer): TAnalogRange;
    property Ranges[Index: Integer]: TAnalogRange read GetRange;
    property DeviceName: string read FDeviceName;
    property IndexDevice: Integer read FIndexDevice;
    property IndexSubdevice: Integer read FIndexSubdevice;
    property NumberOfChannels: Integer read FNumberOfChannels;
    Constructor Create(ADeviceName: string; AIndexDevice, AIndexSubdevice, ANumberOfChannels: Integer);
  end;
//--End--------------------------------------------

var
  frmMain: TfrmMain;

implementation

uses meError, uAbout;

{$R *.dfm}

const
  ME_IDS_NOTOPENED = -1;

//----------------------------------------------
{TAISubDevice}
constructor TAiSubdevice.Create(ADeviceName: string; AIndexDevice, AIndexSubdevice, ANumberOfChannels: Integer);
begin
  inherited Create(TAnalogRange);
  FIndexDevice := AIndexDevice;
  FIndexSubdevice := AIndexSubdevice;
  FNumberOfChannels := ANumberOfChannels;
  FDeviceName := ADeviceName;
end;

function TAiSubdevice.AddRange(AUnit: Integer; APhysicalMin, APhysicalMax: Double; ADigitalMax: Integer): TAnalogRange;
begin
  Result := inherited Add as TAnalogRange;
  Result.FRangeUnit := AUnit;
  Result.FDigitalMax := ADigitalMax;
  Result.FPhysicalMin := APhysicalMin;
  Result.FPhysicalMax := APhysicalMax;
  Result.SetDisplayText;
end;

function TAiSubdevice.GetRange(Index: Integer): TAnalogRange;
begin
  Result := TAnalogRange(inherited GetItem(Index));
end;

procedure TAnalogRange.SetDisplayText;
var
  unitstr0: string;
  unitStr1: string;
  unitStr2: string;
  valStr0: string;
  valStr1: string;
  tmpVal: Double;
begin
  case FRangeUnit of
    ME_UNIT_AMPERE: unitStr2 := 'A';
    ME_UNIT_VOLT:  unitStr2 := 'V';
  end;

  tmpVal := Abs(FPhysicalMin);
  if tmpVal = 0 then
  begin
    unitstr0 := '';
    valStr0 := '0';
  end else if tmpVal < 0.001 then
  begin
    unitStr0 := '';
    valStr0 := FloatToStr(FPhysicalMin * 1000000);
  end else if tmpVal < 1 then
  begin
    unitStr0 := 'm';
    valStr0 := FloatToStr(FPhysicalMin * 1000);
  end else
  begin
    unitstr0 := '';
    valStr0 := FloatToStr(RoundPrecise(FPhysicalMin, 2));
  end;

  tmpVal := Abs(FPhysicalMax);
  if tmpVal = 0 then
  begin
    unitStr1 := '';
    valStr1 := '0';
  end else if tmpVal < 0.001 then
  begin
    unitStr1 := '';
    valStr1 := FloatToStr(FPhysicalMax * 1000000);
  end else if tmpVal < 1 then
  begin
    unitStr1 := 'm';
    valStr1 := FloatToStr(FPhysicalMax * 1000);
  end else
  begin
    unitStr1 := '';
    valStr1 := FloatToStr(RoundPrecise(FPhysicalMax, 2));
  end;
  FDisplayRange := valStr0 + unitstr0 + unitStr2 + '...' + valStr1 + unitStr1 + unitStr2;
  FDisplayUnit := unitStr2;
end;

function TAnalogRange.RoundPrecise(Value: Double; Decimals: Integer): Double;
var
  j: Integer;
  A: Double;
begin
  //round value to given decimal digits
  A := 1;
  case Decimals of
    0: A := 1;
    1: A := 10;
    else
      for j := 1 to Decimals do
        A := A * 10;
  end;
  Result := Round((Value * A) + 0.5) / A;
end;
//----------------------------------------------

procedure TfrmMain.FormCreate(Sender: TObject);
begin
  openErrSuccess := ME_IDS_NOTOPENED;
  lineNumber := 0;
  subDeviceList := TList.Create;
end;

procedure TfrmMain.FormShow(Sender: TObject);
begin
  OpenMEiDSDriver();
end;

procedure TfrmMain.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  CloseMEiDSDriver();
  subDeviceList.Free;
end;

procedure TfrmMain.OpenMEiDSDriver();
var
  numDevices: Integer;
begin
  memoInfo.Lines.Append('Opening Main driver...');
  openErrSuccess := meOpen(ME_OPEN_NO_FLAGS);

  if openErrSuccess = ME_ERRNO_SUCCESS then
  begin
    DisplayVersionLibrary;
    DisplayVersionMainDriver;
    memoInfo.Lines.Append('');

    meQueryNumberDevices(numDevices);
    memoInfo.Lines.Append('Number of devices found: ' + IntToStr(numDevices));

    GetAiSubdevices(numDevices);
  end else
  begin
    DisplayErrorMessage('Error opening Main driver. Error: ', openErrSuccess);
  end;
end;

procedure TfrmMain.GetAiSubdevices(numDevices: Integer);
var
  meError: Integer;
  idxDevice: Integer;
  plugged: Integer;
begin
  if (openErrSuccess = ME_ERRNO_SUCCESS) and (numDevices > 0) then
  begin
    memoInfo.Lines.Add('Retrieving sub-devices of type ME_TYPE_AI (Analog In)...');
    for idxDevice := 0 to numDevices - 1 do
    begin
      meError := GetDevicePlugged(idxDevice, plugged);

      if (meError = ME_ERRNO_SUCCESS) and (plugged = ME_PLUGGED_IN) then
      begin
        GetSubdevicesByType(idxDevice, ME_TYPE_AI);
      end else
      begin
        memoInfo.Lines.Append('Device index ' + IntToStr(idxDevice) + ' not plugged in.');
      end;
    end;
  end;
  DisplaySubdevices();
end;

procedure TfrmMain.GetSubdevicesByType(idxDevice: Integer; subdeviceType: Integer);
var
  idxSubDevice: Integer;
  numSubDevices: Integer;
  meError: Integer;
  idxMatchedSubdevice: Integer;
  devName: array[0..ME_DEVICE_NAME_MAX_COUNT] of AnsiChar;
begin
  meError := meQueryNumberSubdevices(idxDevice, numSubDevices);
  if meError = ME_ERRNO_SUCCESS then
  begin
    meQueryNameDevice(idxDevice, devName, ME_DEVICE_NAME_MAX_COUNT);
    idxSubDevice := 0;
    while idxSubDevice < numSubDevices do
    begin
      meError := meQuerySubdeviceByType(idxDevice,
                                        idxSubDevice,
                                        subdeviceType,
                                        ME_SUBTYPE_ANY,
                                        idxMatchedSubdevice);

      if meError = ME_ERRNO_SUCCESS then
      begin
        AddAiSubdevice(string(devName), idxDevice, idxMatchedSubdevice, subdeviceType);
        idxSubDevice := idxMatchedSubdevice + 1;
      end else
      begin
        Break;
      end;
    end;
  end;
end;


procedure TfrmMain.AddAiSubdevice(devName: string; idxDevice: Integer; idxSubdevice: Integer; subdeviceType: Integer);
var
  meError: Integer;
  numOfChannels: Integer;
  aiSubdevice: TAiSubdevice;
begin
  meError := meQueryNumberChannels(idxDevice,
                        idxSubdevice,
                        numOfChannels);
  if meError = ME_ERRNO_SUCCESS then
  begin
    aiSubdevice := TAiSubdevice.Create(devName, idxDevice, idxSubdevice, numOfChannels);
    AddAnalogRanges(aiSubdevice);
    subDeviceList.Add(aiSubdevice);

    coBoxSubDevs.Items.Append(string(devName)
                              + ',  Device ' + IntToStr(idxDevice)
                              + ',  Subdevice ' + IntToStr(idxSubdevice)
                              + ',  Type: ' + GetSubDevTypeName(subdeviceType)
                              + ',  Channels: ' + IntToStr(numOfChannels)
                              + ',  Ranges: ' + IntToStr(aiSubdevice.Count));
  end;
end;

procedure TfrmMain.AddAnalogRanges(aiSubdevice: TAiSubdevice);
var
  meError: Integer;
  idxRange: Integer;
  numOfRanges: Integer;
begin
  meError := meQueryNumberRanges(aiSubdevice.IndexDevice,
                                 aiSubdevice.IndexSubdevice,
                                 ME_UNIT_ANY,
                                 numOfRanges);

  if meError = ME_ERRNO_SUCCESS then
  begin
    idxRange := 0;
    while idxRange < numOfRanges do
    begin
      AddAnalogRange(aiSubdevice, idxRange);
      Inc(idxRange);
    end;
  end;
end;

procedure TfrmMain.AddAnalogRange(aiSubdevice: TAiSubdevice; idxRange: Integer);
var
  meError: Integer;
  physMax: Double;
  iUnit: Integer;
  physMin: Double;
  digitalMax: Integer;
begin
  meError := meQueryRangeInfo(aiSubdevice.IndexDevice,
                              aiSubdevice.IndexSubdevice,
                              idxRange,
                              iUnit,
                              physMin,
                              physMax,
                              digitalMax);
  if meError = ME_ERRNO_SUCCESS then
  begin
    aiSubdevice.AddRange(iUnit, physMin, physMax, digitalMax);
  end;
end;

procedure TfrmMain.FillSubdeviceChannels(aiSubdevice: TAiSubdevice);
var
  i: Integer;
begin
  coBoxChannel.Clear;
  for i := 0 to aiSubdevice.NumberOfChannels - 1 do
  begin
    coBoxChannel.Items.Add('Channel ' + IntToStr(i));
  end;
  if coBoxChannel.Items.Count > 0 then coBoxChannel.ItemIndex := 0;
end;

procedure TfrmMain.FillSubdeviceRanges(aiSubdevice: TAiSubdevice);
var
  i: Integer;
begin
  coBoxRange.Clear;
  for i := 0 to aiSubdevice.Count - 1 do
  begin
    coBoxRange.Items.Add('Range ' + IntToStr(i) + ': ' + aiSubdevice.Ranges[i].DisplayRange);
  end;
  if coBoxRange.Items.Count > 0 then coBoxRange.ItemIndex := 0;
end;

procedure TfrmMain.coBoxSubdevsChange(Sender: TObject);
begin
  FillSubdeviceProperties(coBoxSubDevs.ItemIndex);
end;

procedure TfrmMain.FillSubdeviceProperties(idxSelectedSubdev: Integer);
var
  aiSubdevice: TAiSubdevice;
begin
  if idxSelectedSubdev >= 0 then
  begin
    aiSubdevice := TAiSubdevice(subDeviceList[idxSelectedSubdev]);
    FillSubdeviceChannels(aiSubdevice);
    FillSubdeviceRanges(aiSubdevice);
  end;
end;

procedure TfrmMain.btnReadDataClick(Sender: TObject);
var
  meError: Integer;
  idxRange: Integer;
  valDigital: Integer;
  valPhysical: Double;
  aiSubdevice: TAiSubdevice;
begin
  if coBoxSubDevs.ItemIndex >= 0 then
  begin
    aiSubdevice := TAiSubdevice(subDeviceList[coBoxSubdevs.ItemIndex]);
    idxRange := coBoxRange.ItemIndex;

    meError := ConfigureSubdeviceForInput(aiSubdevice, coBoxChannel.ItemIndex, idxRange);
    if meError = ME_ERRNO_SUCCESS then
    begin

      meError := ReadSingleValue(aiSubdevice, coBoxChannel.ItemIndex, valDigital);
      if meError = ME_ERRNO_SUCCESS then
      begin
        meError := ConvertDigitalToPhysical(aiSubdevice, idxRange, valDigital, valPhysical);

        if meError = ME_ERRNO_SUCCESS then
        begin
          DisplayValue(aiSubdevice, idxRange, valDigital, valPhysical);
        end else
        begin
          DisplayErrorMessage('meUtilityDigitalToPhysical - Error: ', meError);
        end;
      end else
      begin
        DisplayErrorMessage('meIOSingle - Error: ', meError);
      end;
    end else
    begin
      DisplayErrorMessage('meIOSingleConfig - Error: ', meError);
    end;
  end;
end;

function TfrmMain.ConfigureSubdeviceForInput(aiSubdevice: TAiSubdevice; idxChannel, idxRange: Integer): Integer;
begin
  Result := meIOSingleConfig(aiSubdevice.IndexDevice,
                             aiSubdevice.IndexSubdevice,
                             idxChannel,
                             idxRange,
                             ME_REF_AI_GROUND,
                             ME_TRIG_CHAN_DEFAULT,
                             ME_TRIG_TYPE_SW,
                             ME_VALUE_NOT_USED,
                             ME_IO_SINGLE_CONFIG_NO_FLAGS);
end;

function TfrmMain.ConvertDigitalToPhysical(aiSubdevice: TAiSubdevice; idxRange: Integer;
  value: Integer; var valAnalog: Double): Integer;
var
  dPhysMax: Double;
  dPhysMin: Double;
  valDigitalMax: Integer;
begin
  dPhysMin := aiSubdevice.Ranges[idxRange].PhysicalMin;
  dPhysMax := aiSubdevice.Ranges[idxRange].PhysicalMax;
  valDigitalMax := aiSubdevice.Ranges[idxRange].DigitalMax;
  Result := meUtilityDigitalToPhysical(dPhysMin, dPhysMax, valDigitalMax, value, ME_VALUE_NOT_USED, 0, valAnalog);
end;

procedure TfrmMain.DisplayValue(aiSubdevice: TAiSubdevice; idxRange: Integer; valDigital: Integer; valPhysical: Double);
begin
  Inc(lineNumber);
  memoInfo.lines.add(Format('%.2d', [lineNumber])
                     + ' Read from ' + aiSubdevice.DeviceName
                     + ',  Device: ' + IntToStr(aiSubdevice.IndexDevice)
                     + ',  Subdevice: ' + IntToStr(aiSubdevice.IndexSubdevice)
                     + ',  Digital value: ' + IntToStr(valDigital)
                     + ',  Physical value: '
                     + Format('%-12.7f', [valPhysical])
                     + ' ' + aiSubdevice.Ranges[idxRange].DisplayUnit);
end;

function TfrmMain.ReadSingleValue(aiSubdevice: TAiSubdevice; idxChannel: Integer; var value: Integer): Integer;
var
  meError: Integer;
  io_single: array[0..0] of TMeIOSingle;
begin
  io_single[0].iDevice:= aiSubdevice.IndexDevice;
  io_single[0].iSubdevice := aiSubdevice.IndexSubdevice;
  io_single[0].iChannel := idxChannel;
  io_single[0].iDir := ME_DIR_INPUT;
  io_single[0].iValue := 0;
  io_single[0].iTimeOut := 0;
  io_single[0].iFlags := ME_IO_SINGLE_TYPE_NO_FLAGS;
  io_single[0].iErrno := 0;

  meError := meIOSingle(io_single[0],
                        1,
                        ME_IO_SINGLE_NO_FLAGS);
  value := io_single[0].iValue;
  Result := meError;
end;

procedure TfrmMain.CleanupSubdeviceCollection();
var
  i: Integer;
  subDev: TAiSubdevice;
begin
  for i := 0 to subDeviceList.Count -1 do
  begin
    subDev := TAiSubdevice(subDeviceList[i]);
    subDev.Free;
  end;
  subDeviceList.Clear;
end;

procedure TfrmMain.DisplayErrorMessage(ErrMsg: string; meError: Integer);
var
  errorMsg: array[0..ME_ERROR_MSG_MAX_COUNT] of AnsiChar;
begin
  meErrorGetMessage(meError, errorMsg, ME_ERROR_MSG_MAX_COUNT);
  memoInfo.lines.add(ErrMsg + IntToStr(meError));
  memoInfo.Lines.Add('ErrorMessage: ' + errorMsg);
end;

procedure TfrmMain.btnClearOutputClick(Sender: TObject);
begin
  memoInfo.Clear;
  lineNumber := 0;
end;

procedure TfrmMain.btnCloseClick(Sender: TObject);
begin
  Close;
end;

procedure TfrmMain.CloseMEiDSDriver();
begin
  if openErrSuccess = ME_ERRNO_SUCCESS then
  begin
    meClose(ME_CLOSE_NO_FLAGS);
    openErrSuccess := ME_IDS_NOTOPENED;
    CleanupSubdeviceCollection;
    coBoxChannel.Clear;
    coBoxRange.Clear;
    coBoxSubDevs.Clear;
    btnReadData.Enabled := False;
    btnReset.Enabled := False;
  end;
end;

procedure TfrmMain.btnResetClick(Sender: TObject);
var
  meError: Integer;
  aiSubdevice: TAiSubdevice;
begin
  if (openErrSuccess = ME_ERRNO_SUCCESS) and (coBoxSubDevs.ItemIndex >= 0) then
  begin
    aiSubdevice := TAiSubdevice(subDeviceList[coBoxSubDevs.ItemIndex]);

    meError := meIOResetSubdevice(aiSubdevice.IndexDevice,
                                  aiSubdevice.IndexSubdevice,
                                  ME_IO_RESET_SUBDEVICE_NO_FLAGS);
    if meError = ME_ERRNO_SUCCESS then
    begin
      coBoxChannel.ItemIndex := 0;
      coBoxRange.ItemIndex := 0;
      memoInfo.Lines.Add('Reset Device ' + IntToStr(aiSubdevice.IndexDevice) + ', Subdevice ' + IntToStr(aiSubdevice.IndexSubdevice));
    end else
    begin
      DisplayErrorMessage('meIOResetSubdevice - Error: ', meError);
    end;
  end;
end;

function TfrmMain.GetSubDevTypeName(subdeviceType: Integer): string;
begin
  case subdeviceType of
    ME_TYPE_INVALID: Result := 'ME_TYPE_INVALID';
    ME_TYPE_AI: Result := 'ME_TYPE_AI';
    ME_TYPE_AO: Result := 'ME_TYPE_AO';
    ME_TYPE_DIO: Result := 'ME_TYPE_DIO';
    ME_TYPE_DO: Result := 'ME_TYPE_DO';
    ME_TYPE_DI: Result := 'ME_TYPE_DI';
    ME_TYPE_CTR: Result := 'ME_TYPE_CTR';
    ME_TYPE_EXT_IRQ: Result := 'ME_TYPE_EXT_IRQ';
    ME_TYPE_FIO: Result := 'ME_TYPE_FIO';
    ME_TYPE_FO: Result := 'ME_TYPE_FO';
    ME_TYPE_FI: Result := 'ME_TYPE_FI';
    ME_TYPE_FPGA: Result := 'ME_TYPE_FPGA';
  else
    Result := '$' + IntToHex(subdeviceType, 8);
  end;
end;

procedure TfrmMain.DisplaySubdevices();
begin
  if coBoxSubDevs.Items.Count > 0 then
  begin
    memoInfo.Lines.Append('Number of AI sub-devices found: ' + IntToStr(coBoxSubdevs.Items.Count));
    memoInfo.Lines.AddStrings(coBoxSubdevs.Items);
    memoInfo.Lines.Add('');
    coBoxSubdevs.ItemIndex := 0;
    coBoxSubdevsChange(coBoxSubdevs);
    btnReadData.Enabled := True;
    btnReset.Enabled := True;
  end else
  begin
    memoInfo.Lines.Append('No AI sub-devices found.');
    CloseMEiDSDriver;
  end;
end;

function TfrmMain.GetDevicePlugged(idxDevice: Integer; var plugged: Integer): Integer;
var
  busNo: Integer;
  vendorId: Integer;
  deviceId: Integer;
  funcNo: Integer;
  busType: Integer;
  serialNo: Integer;
  devNo: Integer;
begin
  Result := meQueryInfoDevice(idxDevice,
                    vendorId,
                    deviceId,
                    serialNo,
                    busType,
                    busNo,
                    devNo,
                    funcNo,
                    plugged);
end;

procedure TfrmMain.DisplayVersionLibrary;
var
  ver: Integer;
begin
  meQueryVersionLibrary(ver);
  memoInfo.Lines.Append('Library version: ' + MeVersionToString(ver));
end;

procedure TfrmMain.DisplayVersionMainDriver;
var
  ver: Integer;
begin
  meQueryVersionMainDriver(ver);
  memoInfo.Lines.Append('Main driver version: ' + MeVersionToString(ver));
end;

function TfrmMain.MeVersionToString(version: Integer): string;
begin
  Result := Format('%d.%d.%d.%d', [version shr 24, $FF and (version shr 16), $FF and (version shr 8), $FF and version]);
end;

procedure TfrmMain.btnAboutClick(Sender: TObject);
begin
  frmAbout := TfrmAbout.Create(Self);
  try
    frmAbout.PopupParent := Self;
    frmAbout.ShowModal;
  finally
    frmAbout.Free;
  end;
end;

end.
