//==============================================================================
//Copyright(c), Meilhaus Electronic GmbH
//              http://www.meilhaus.de
//              support@meilhaus.de
//
//Source File : uMain.pas
//Description : Example program for reading single values from DI and DIO devices
//              - Open ME-iDS main driver
//              - Go through the list of all devices and find DI and DIO subdevices
//              - Add DI and DIO subdevices to the subdevice combobox
//              - Read data from selected subdevice when user clicks on button "Read Data"
//              - Data are always read from channel 0 in this example
//              - Close ME-iDS main driver when program is closed
//==============================================================================
unit uMain;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, ExtCtrls, Math, StrUtils, meIDS;

type
//-------------------------------------------------
{TDiSubdevice}
  TDiSubdevice = class(TCollectionItem)
  private
    FDeviceName: string;
    FIndexDevice: Integer;
    FIndexSubdevice: Integer;
    FNumberOfChannels: Integer;
    FSingleConfigFlag: Integer;
    FSingleTypeFlag: Integer;
  published
    property DeviceName: string read FDeviceName;
    property IndexDevice: Integer read FIndexDevice;
    property IndexSubdevice: Integer read FIndexSubdevice;
    property NumberOfChannels: Integer read FNumberOfChannels;
    property SingleConfigFlag: Integer read FSingleConfigFlag;
    property SingleTypeFlag: Integer read FSingleTypeFlag;
  end;

  TDiSubdevices = class(TCollection)
  private
    function GetSubdevice(Index: Integer): TDiSubdevice;
  public
    function AddSubdevice(ADeviceName: string;
                          AIdxDevice: Integer;
                          AIdxSubdevice: Integer;
                          ANumOfChannels: Integer): TDiSubdevice;
    property Items[Index: Integer]: TDiSubdevice read GetSubdevice; default;
    Constructor Create;
  end;
//-------------------------------------------------
{TfrmMain}
  TfrmMain = class(TForm)
    memoInfo: TMemo;
    btnClose: TButton;
    btnDeviceRead: TButton;
    lbl1: TLabel;
    coBoxSubDevs: TComboBox;
    lbl3: TLabel;
    btnClearHwInfo: TButton;
    grpBox1: TGroupBox;
    btnAbout: TButton;
    btnReset: TButton;
    procedure btnCloseClick(Sender: TObject);
    procedure btnDeviceReadClick(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure FormCreate(Sender: TObject);
    procedure btnClearHwInfoClick(Sender: TObject);
    procedure btnAboutClick(Sender: TObject);
    procedure btnResetClick(Sender: TObject);
    procedure FormShow(Sender: TObject);
  private
    { Private-Deklarationen }
    openErrSuccess: Integer;
    diSubdevices: TDiSubdevices;
    lineNumber: Integer;
    function MeVersionToString(iVersion: Integer): string;
    procedure GetDiDioSubdevices(numDevices: Integer);
    procedure GetSubdevicesByType(iDevice: Integer; iType: Integer);
    function GetSubDevTypeName(iType: Integer): string;
    procedure AddDiSubdevice(devName: string; iDevice: Integer; iSubdevice: Integer; subdeviceType: Integer);
    function IntToBin(val: Cardinal; dispDigits: Integer): string;
    procedure OpenMEiDSDriver();
    procedure CloseMEiDSDriver();
    procedure DisplayErrorMessage(ErrMsg: string; meError: Integer);
    procedure DisplayVersionLibrary();
    procedure DisplayVersionMainDriver();
    function GetDevicePlugged(idxDevice: Integer; var plugged: Integer): Integer;
    procedure DisplaySubdevices();
    function ConfigureSubdeviceForInput(diSubdevice: TDiSubdevice): Integer;
    function ReadSingleValue(diSubdevice: TDiSubdevice; var value: Integer): Integer;
    procedure DisplayValue(diSubdevice: TDiSubdevice; value: Integer);
  public
    { Public-Deklarationen }
  end;

var
  frmMain: TfrmMain;


implementation

uses meError, uAbout;

{$R *.dfm}

const
  ME_IDS_NOTOPENED = -1;
  
//-------------------------------------------------
{TBitChDevice}
constructor TDiSubdevices.Create();
begin
  inherited Create(TDiSubdevice);
end;

function TDiSubdevices.GetSubdevice(Index: Integer): TDiSubdevice;
begin
  Result := TDiSubdevice(inherited GetItem(Index));
end;

function TDiSubdevices.AddSubdevice(ADeviceName: string;
                                    AIdxDevice: Integer;
                                    AIdxSubdevice: Integer;
                                    ANumOfChannels: Integer): TDiSubdevice;
begin
  Result := inherited Add as TDiSubdevice;
  Result.FDeviceName := ADeviceName;
  Result.FIndexDevice := AIdxDevice;
  Result.FIndexSubdevice := AIdxSubdevice;
  Result.FNumberOfChannels := ANumOfChannels;

  case ANumOfChannels of
    1:
    begin
       Result.FSingleConfigFlag := ME_IO_SINGLE_CONFIG_DIO_BIT;
       Result.FSingleTypeFlag := ME_IO_SINGLE_TYPE_DIO_BIT;
    end;
    8:
    begin
      Result.FSingleConfigFlag := ME_IO_SINGLE_CONFIG_DIO_BYTE;
      Result.FSingleTypeFlag := ME_IO_SINGLE_TYPE_DIO_BYTE;
    end;
    16:
    begin
      Result.FSingleConfigFlag := ME_IO_SINGLE_CONFIG_DIO_WORD;
      Result.FSingleTypeFlag := ME_IO_SINGLE_TYPE_DIO_WORD;
    end;
    24:
    begin
      Result.FSingleConfigFlag := ME_IO_SINGLE_CONFIG_DIO_DWORD;
      Result.FSingleTypeFlag := ME_IO_SINGLE_TYPE_DIO_DWORD;
    end;
    32:
    begin
      Result.FSingleConfigFlag := ME_IO_SINGLE_CONFIG_DIO_DWORD;
      Result.FSingleTypeFlag := ME_IO_SINGLE_TYPE_DIO_DWORD;
    end;
  else
    Result.FSingleConfigFlag := ME_VALUE_INVALID;
    Result.FSingleTypeFlag := ME_VALUE_INVALID;
  end;
end;

//----------------------------------------------
procedure TfrmMain.FormCreate(Sender: TObject);
begin
  openErrSuccess := ME_IDS_NOTOPENED;
  lineNumber := 0;
  diSubdevices := TDiSubdevices.Create;
end;

procedure TfrmMain.FormShow(Sender: TObject);
begin
  OpenMEiDSDriver();
end;

procedure TfrmMain.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  CloseMEiDSDriver();
  diSubdevices.Free;
end;

procedure TfrmMain.OpenMEiDSDriver();
var
  numDevices: Integer;
begin
  memoInfo.Lines.Append('Opening Main driver...');
  openErrSuccess := meOpen(ME_OPEN_NO_FLAGS);

  if openErrSuccess = ME_ERRNO_SUCCESS then
  begin
    DisplayVersionLibrary;
    DisplayVersionMainDriver;
    memoInfo.Lines.Append('');

    meQueryNumberDevices(numDevices);
    memoInfo.Lines.Append('Number of devices found: ' + IntToStr(numDevices));

    GetDiDioSubdevices(numDevices);
  end else
  begin
    DisplayErrorMessage('Error opening Main driver. Error: ', openErrSuccess);
  end;
end;

procedure TfrmMain.GetDiDioSubdevices(numDevices: Integer);
var
  meError: Integer;
  idxDevice: Integer;
  plugged: Integer;
begin
  if openErrSuccess = ME_ERRNO_SUCCESS then
  begin
    memoInfo.Lines.Add('Retrieving sub-devices of type ME_TYPE_DI (Digital In) and ME_TYPE_DIO (Digital In/Out)...');
    for idxDevice := 0 to numDevices - 1 do
    begin
      meError := GetDevicePlugged(idxDevice, plugged);

      if (meError = ME_ERRNO_SUCCESS) and (plugged = ME_PLUGGED_IN) then
      begin
        GetSubdevicesByType(idxDevice, ME_TYPE_DI);
        GetSubdevicesByType(idxDevice, ME_TYPE_DIO);
      end else
      begin
        memoInfo.Lines.Append('Device ' + IntToStr(idxDevice) + ' not plugged in.');
      end;
    end;
  end;
  DisplaySubdevices();
end;

procedure TfrmMain.GetSubdevicesByType(iDevice: Integer; iType: Integer);
var
  idxSubDevice: Integer;
  numSubDevices: Integer;
  meError: Integer;
  idxMatchedSubDevice: Integer;
  devName: array[0..ME_DEVICE_NAME_MAX_COUNT] of AnsiChar;
begin
  meError := meQueryNumberSubdevices(iDevice, numSubDevices);
  if meError = ME_ERRNO_SUCCESS then
  begin
    meQueryNameDevice(iDevice, devName, ME_DEVICE_NAME_MAX_COUNT);
    idxSubDevice := 0;
    while idxSubDevice < numSubDevices do
    begin
      meError := meQuerySubdeviceByType(iDevice,
                                        idxSubDevice,
                                        iType,
                                        ME_SUBTYPE_ANY,
                                        idxMatchedSubDevice);

      if meError = ME_ERRNO_SUCCESS then
      begin
        AddDiSubdevice(string(devName), iDevice, idxMatchedSubDevice, iType);
        idxSubDevice := idxMatchedSubDevice + 1;
      end else
      begin
        Break;
      end;
    end;
  end;
end;

procedure TfrmMain.AddDiSubdevice(devName: string; iDevice: Integer; iSubdevice: Integer; subdeviceType: Integer);
var
  numOfChannels: Integer;
begin
  meQueryNumberChannels(iDevice, iSubdevice, numOfChannels);
  diSubdevices.AddSubdevice(devName, iDevice, iSubdevice, numOfChannels);

  coBoxSubDevs.Items.Append(devName
                            + ',  Device ' + IntToStr(iDevice)
                            + ',  Subdevice ' + IntToStr(iSubdevice)
                            + ',  Type: ' + GetSubDevTypeName(subdeviceType)
                            + ',  Channels: ' + IntToStr(numOfChannels));
end;

procedure TfrmMain.btnDeviceReadClick(Sender: TObject);
var
  meError: Integer;
  value: Integer;
  diSubdevice: TDiSubdevice;
begin
  if coBoxSubDevs.ItemIndex >= 0 then
  begin
    diSubdevice := diSubdevices[coBoxSubDevs.ItemIndex];

    meError := ConfigureSubdeviceForInput(diSubdevice);
    if meError = ME_ERRNO_SUCCESS then
    begin

      meError := ReadSingleValue(diSubdevice, value);
      if meError = ME_ERRNO_SUCCESS then
      begin
        DisplayValue(diSubdevice, value);
      end else
      begin
        DisplayErrorMessage('meIOSingle - Error: ', meError);
      end;
    end else
    begin
      DisplayErrorMessage('meIOSingleConfig - Error: ', meError);
    end;
  end;
end;

function TfrmMain.ConfigureSubdeviceForInput(diSubdevice: TDiSubdevice): Integer;
begin
  Result := meIOSingleConfig(diSubdevice.IndexDevice,
                              diSubdevice.IndexSubdevice,
                              0,                          // Channel index
                              ME_SINGLE_CONFIG_DIO_INPUT,
                              ME_REF_NONE,
                              ME_TRIG_CHAN_NONE,
                              ME_TRIG_TYPE_NONE,
                              ME_TRIG_EDGE_NONE,
                              diSubdevice.SingleConfigFlag);
end;

function TfrmMain.ReadSingleValue(diSubdevice: TDiSubdevice; var value: Integer): Integer;
var
  meError: Integer;
  io_single: array[0..0] of TMeIOSingle;
begin
  io_single[0].iDevice:= diSubdevice.IndexDevice;
  io_single[0].iSubDevice := diSubdevice.IndexSubdevice;
  io_single[0].iChannel := 0;
  io_single[0].iDir := ME_DIR_INPUT;
  io_single[0].iValue := 0;
  io_single[0].iTimeOut := 0;
  io_single[0].iFlags := diSubdevice.SingleTypeFlag;
  io_single[0].iErrno := 0;

  meError := meIOSingle(io_single[0],
                        1,
                        ME_IO_SINGLE_NO_FLAGS);
  value := io_single[0].iValue;
  Result := meError;
end;

procedure TfrmMain.DisplayValue(diSubdevice: TDiSubdevice; value: Integer);
begin
  inc(lineNumber);
  memoInfo.lines.add(Format('%.2d', [lineNumber])
                     + ' Read from ' + diSubdevice.DeviceName
                     + ',  Device: ' + IntToStr(diSubdevice.IndexDevice)
                     + ',  Subdevice: ' + IntToStr(diSubdevice.IndexSubdevice)
                     + ',  Value hex: ' + IntToHex(value, diSubdevice.NumberOfChannels div 4)
                     + ',  bin: ' + IntToBin(value, diSubdevice.NumberOfChannels)
                     + ',  dec: ' + IntToStr(value));
end;

procedure TfrmMain.btnResetClick(Sender: TObject);
var
  meError: Integer;
  diSubdevice: TDiSubdevice;
begin
  if (openErrSuccess = ME_ERRNO_SUCCESS) and (coBoxSubDevs.ItemIndex >= 0) then
  begin
    diSubdevice := diSubdevices[coBoxSubDevs.ItemIndex];

    meError := meIOResetSubdevice(diSubdevice.IndexDevice,
                                  diSubdevice.IndexSubdevice,
                                  ME_IO_RESET_SUBDEVICE_NO_FLAGS);
    if meError = ME_ERRNO_SUCCESS then
    begin
      memoInfo.Lines.Add('Reset Device ' + IntToStr(diSubdevice.IndexDevice) + ', Subdevice ' + IntToStr(diSubdevice.IndexSubdevice));
    end else
    begin
      DisplayErrorMessage('meIOResetSubdevice - Error: ', meError);
    end;
  end;
end;

procedure TfrmMain.DisplayErrorMessage(ErrMsg: string; meError: Integer);
var
  errorMsg: array[0..ME_ERROR_MSG_MAX_COUNT] of AnsiChar;
begin
  meErrorGetMessage(meError, errorMsg, ME_ERROR_MSG_MAX_COUNT);
  memoInfo.lines.add(ErrMsg + IntToStr(meError));
  memoInfo.Lines.Add('ErrorMessage: ' + errorMsg);
end;

function TfrmMain.GetSubDevTypeName(iType: Integer): string;
begin
  case iType of
    ME_TYPE_INVALID: Result := 'ME_TYPE_INVALID';
    ME_TYPE_AI: Result := 'ME_TYPE_AI';
    ME_TYPE_AO: Result := 'ME_TYPE_AO';
    ME_TYPE_DIO: Result := 'ME_TYPE_DIO';
    ME_TYPE_DO: Result := 'ME_TYPE_DO';
    ME_TYPE_DI: Result := 'ME_TYPE_DI';
    ME_TYPE_CTR: Result := 'ME_TYPE_CTR';
    ME_TYPE_EXT_IRQ: Result := 'ME_TYPE_EXT_IRQ';
    ME_TYPE_FIO: Result := 'ME_TYPE_FIO';
    ME_TYPE_FO: Result := 'ME_TYPE_FO';
    ME_TYPE_FI: Result := 'ME_TYPE_FI';
    ME_TYPE_FPGA: Result := 'ME_TYPE_FPGA';
  else
    Result := '$' + IntToHex(iType, 8);
  end;
end;

procedure TfrmMain.DisplayVersionLibrary;
var
  ver: Integer;
begin
  meQueryVersionLibrary(ver);
  memoInfo.Lines.Append('Library version: ' + MeVersionToString(ver));
end;

procedure TfrmMain.DisplayVersionMainDriver;
var
  ver: Integer;
begin
  meQueryVersionMainDriver(ver);
  memoInfo.Lines.Append('Main driver version: ' + MeVersionToString(ver));
end;

function TfrmMain.GetDevicePlugged(idxDevice: Integer; var plugged: Integer): Integer;
var
  busNo: Integer;
  vendorId: Integer;
  deviceId: Integer;
  funcNo: Integer;
  busType: Integer;
  serialNo: Integer;
  devNo: Integer;
begin
  Result := meQueryInfoDevice(idxDevice,
                    vendorId,
                    deviceId,
                    serialNo,
                    busType,
                    busNo,
                    devNo,
                    funcNo,
                    plugged);
end;

procedure TfrmMain.DisplaySubdevices();
begin
  if coBoxSubDevs.Items.Count > 0 then
  begin
    memoInfo.Lines.Append('Number of DI or DIO sub-devices found: ' + IntToStr(coBoxSubDevs.Items.Count));
    memoInfo.Lines.Add('');
    memoInfo.Lines.AddStrings(coBoxSubDevs.Items);
    memoInfo.Lines.Add('');
    coBoxSubDevs.ItemIndex := 0;
    btnDeviceRead.Enabled := True;
    btnReset.Enabled := True;
  end else
  begin
    memoInfo.Lines.Append('No DI or DIO sub-devices found.');
    CloseMEiDSDriver;
  end;
end;

function TfrmMain.IntToBin(val: Cardinal; dispDigits: Integer): string;
var
  i: Integer;
  b: Integer;
  tmpStr: string;
const
  binStr: array[0..15] of string = ('0000', '0001', '0010', '0011',
                                    '0100', '0101', '0110', '0111',
                                    '1000', '1001', '1010', '1011',
                                    '1100', '1101', '1110', '1111');
begin
  tmpStr := '';
  if dispDigits in [1, 8, 16, 24, 32] then
  begin
    if dispDigits = 1 then
    begin
      tmpStr := IntToStr(val and 1);
    end else
    begin
      for i := (dispDigits shr 2) -1 downto 0 do
      begin
        b := (val shr (i shl 2)) and $0F;
        tmpStr := tmpStr + binStr[b];
      end;
    end;
  end;
  Result := tmpStr;
end;

function TfrmMain.MeVersionToString(iVersion: Integer): string;
begin
  Result := Format('%d.%d.%d.%d', [iVersion shr 24, $FF and (iVersion shr 16), $FF and (iVersion shr 8), $FF and iVersion]);
end;

procedure TfrmMain.btnClearHwInfoClick(Sender: TObject);
begin
  memoInfo.Clear;
  lineNumber := 0;
end;

procedure TfrmMain.btnCloseClick(Sender: TObject);
begin
  Close;
end;

procedure TfrmMain.CloseMEiDSDriver();
begin
  if openErrSuccess = ME_ERRNO_SUCCESS then
  begin
    meClose(ME_CLOSE_NO_FLAGS);
    openErrSuccess := ME_IDS_NOTOPENED;
    coBoxSubDevs.Clear;
    diSubdevices.Clear;
    btnDeviceRead.Enabled := False;
    btnReset.Enabled := False;
  end;
end;

procedure TfrmMain.btnAboutClick(Sender: TObject);
begin
  frmAbout := TfrmAbout.Create(Self);
  try
    frmAbout.PopupParent := Self;
    frmAbout.ShowModal;
  finally
    frmAbout.Free;
  end;
end;

end.
